##################################################################################################################################
# ; ВНИМАНИЕ! При добавлении новых тегов убедитесь в работоспособности утилиты. Если всё хорошо, закоммитьте!
# ; Для создания или правки тега сделайте запись по образцу:
# ; [XXXX]            // номер тега
# ; Type =  XXXX      // тип тега
# ; Head =            // не используется (пока)
# ; Name =  XXXX      // введите название тега
# ;
# ; Используйте следующие типы:
# ; BYTE	     - целое число 1 байт
# ; INT16        - целое число 2 байта
# ; INT32        - целое число 4 байта
# ; VLN          - байты представляют собой число, LE
# ; FPD          - фискальный признак, последние 4 байта с конца
# ; STRING       - байты представляют собой символы
# ; COINS        - цена в копейках, при выводе число делится на 100 (в рубли), байты представляют собой число, LE
# ; UNIXTIME     - дата и время в формате UnixTime
# ; FVLN         - дробное число, первый байт - позиция запятой, остальные байты представляют собой число, LE, применяется для количества
# ; STLV         - тег представляет собой структуру STLV
# ; ENUM или SET - множество значений в зависимости от числа, байты представляют собой число, LE
# ;      Для использования ENUM или SET необходимо также задать значения (см. пример тег 1209):
# ;      [XXXX]
# ;      Type =  ENUM
# ;      Head =
# ;      Name =  XXXX
# ;      Value1 = Значение 1 (соотв. числу 1)
# ;      Value2 = Значение 2 (соотв. числу 2)
# ;      Value7 = Значение 7 (соотв. числу 7)
# ; и.т.п
# ; BITS        - значением является множество, задаваемое битовой маской
# ;      Для использования BITS необходимо также задать значения (см. пример тег 1057):
# ;      [XXXX]
# ;      Type =  ENUM
# ;      Head =
# ;      Name =  XXXX
# ;      BIT0 = Значение 1 (соотв. бит 0 = 1)
# ;      BIT1 = Значение 2 (соотв. бит 1 = 1)
# ;      BIT2 = Значение 3 (соотв. бит 2 = 1)
# BYTES - массив байт
##################################################################################################################################

BYTE = "BYTE"
INT16 = "INT16"
INT32 = "INT32"
VLN = "VLN"
FPD = "FPD"
STRING = "STRING"
COINS = "COINS"
UNIXTIME = "UNIXTIME"
FVLN = "FVLN"
STLV = "STLV"
ENUM = "ENUM"
SET = "SET"
BITS = "BITS"
BYTES = "BYTES"

tags = {
    "6": {
        "Type": STLV,
        "Head": "ОТЧЁТ О ЗАКРЫТИИ ФИСКАЛЬНОГО НАКОПИТЕЛЯ",
        "Name": "отчёт о закрытии фискального накопителя",
    },
    "106": {
        "Type": STLV,
        "Head": "ОТЧЁТ О ЗАКРЫТИИ ФИСКАЛЬНОГО НАКОПИТЕЛЯ",
        "Name": "отчёт о закрытии фискального накопителя",
    },
    "7": {
        "Type": STLV,
        "Head": "ПОДТВЕРЖДЕНИЕ ОПЕРАТОРА",
        "Name": "подтверждение оператора",
    },
    "107": {
        "Type": STLV,
        "Head": "ПОДТВЕРЖДЕНИЕ ОПЕРАТОРА",
        "Name": "подтверждение оператора",
    },
    "1001": {
        "Type": BYTE,
        "Head": "АВТОМАТ.РЕЖИМ:",
        "Name": "автоматический режим",
    },
    "1002": {
        "Type": BYTE,
        "Head": "АВТОНОМН.РЕЖИМ:",
        "Name": "автономный режим",
    },
    "1003": {
        "Type": "",
        "Head": "адрес банковского агента для банковских агентов",
        "Name": "",
    },
    "1004": {
        "Type": "",
        "Head": "адрес для банковского субагента для банковских агентов",
        "Name": "",
    },
    "1005": {
        "Type": STRING,
        "Head": "АДР.ОП.ПЕРЕВОДА:",
        "Name": "адрес оператора по переводу денежных средств для банковских агентов",
    },
    "1006": {
        "Type": "",
        "Head": "адрес платежного агента для платежных агентов",
        "Name": "",
    },
    "1007": {
        "Type": "",
        "Head": "адрес платежного субагента для платежных агентов",
        "Name": "",
    },
    "1008": {
        "Type": STRING,
        "Head": "",
        "Name": "адрес покупателя",
    },
    "1009": {
        "Type": STRING,
        "Head": "",
        "Name": "адрес (место) расчетов",
    },
    "1010": {
        "Type": VLN,
        "Head": "ВОЗН.ПЛ.АГЕНТА:",
        "Name": "размер вознаграждения банковского агента (субагента) для банковских агентов",
    },
    "1011": {
        "Type": "",
        "Head": "",
        "Name": "размер вознаграждения платежного агента (субагента) для платежных агентов",
    },
    "1012": {
        "Type": UNIXTIME,
        "Head": "",
        "Name": "дата, время",
    },
    "1013": {
        "Type": STRING,
        "Head": "ЗН ККТ:",
        "Name": "заводской номер ККТ",
    },
    "1014": {
        "Type": "",
        "Head": "",
        "Name": "значение типа строка",
    },
    "1015": {
        "Type": "",
        "Head": "",
        "Name": "значение типа целое",
    },
    "1016": {
        "Type": STRING,
        "Head": "ИНН.ОП.ПЕРЕВОДА:",
        "Name": "ИНН оператора по переводу денежных средств для банковских агентов",
    },
    "1017": {
        "Type": STRING,
        "Head": "ИНН ОФД:",
        "Name": "ИНН ОФД",
    },
    "1018": {
        "Type": STRING,
        "Head": "ИНН:",
        "Name": "ИНН пользователя",
    },
    "1019": {
        "Type": "",
        "Head": "",
        "Name": "информационное сообщение",
    },
    "1020": {
        "Type": COINS,
        "Head": "ИТОГ:",
        "Name": "ИТОГ",
    },
    "1021": {
        "Type": STRING,
        "Head": "КАССИР:",
        "Name": "кассир",
    },
    "1022": {
        "Type": BYTE,
        "Head": "",
        "Name": "код ответа ОФД",
        "Value0": "успешно",
        "Value11": "неисправимая ошибка, содержание документа не распознано",
        "Value14": "ошибка ФЛК при обработке документа",
    },
    "1023": {
        "Type": FVLN,
        "Head": "",
        "Name": "количество",
    },
    "1024": {
        "Type": STRING,
        "Head": "",
        "Name": "наименование банковского агента для банковских агентов",
    },
    "1025": {
        "Type":STRING,
        "Head": "",
        "Name": "наименование банковского субагента для банковских агентов",
    },
    "1026": {
        "Type": STRING,
        "Head": "ОПЕРАТОР ПЕРЕВОДА:",
        "Name": "наименование оператора по переводу денежных средств для банковских агентов",
    },
    "1027": {
        "Type": STRING,
        "Head": "",
        "Name": "наименование платежного агента для платежных агентов",
    },
    "1028": {
        "Type": STRING,
        "Head": "",
        "Name": "наименование платежного субагента для платежных агентов",
    },
    "1029": {
        "Type": STRING,
        "Head": "",
        "Name": "наименование реквизита",
    },
    "1030": {
        "Type": STRING,
        "Head": "",
        "Name": "наименование товара",
    },
    "1031": {"Type": COINS, "Head": "НАЛИЧНЫМИ:", "Name": "форма расчета – наличными"},
    "1032": {"Type": STRING, "Head": "", "Name": "налог"},
    "1033": {"Type": STRING, "Head": "", "Name": "налоги"},
    "1034": {"Type": STRING, "Head": "", "Name": "наценка (ставка)"},
    "1035": {"Type": STRING, "Head": "", "Name": "наценка (сумма)"},
    "1036": {"Type": STRING, "Head": "АВТОМАТ:", "Name": "номер автомата"},
    "1037": {"Type": STRING, "Head": "РНККТ:", "Name": "регистрационный номер ККТ"},
    "1038": {"Type": INT32, "Head": "СМЕНА:", "Name": "номер смены"},
    "1039": {"Type": STRING, "Head": "", "Name": "зарезервирован"},
    "1040": {
        "Type": INT32,
        "Head": "ФД:",
        "Name": "порядковый номер фискального документа",
    },
    "1041": {
        "Type": STRING,
        "Head": "ФН:",
        "Name": "заводской номер фискального накопителя",
    },
    "1042": {"Type": INT32, "Head": "", "Name": "номер чека за смену"},
    "1043": {
        "Type": COINS,
        "Head": "",
        "Name": "общая стоимость позиции с учетом скидок и наценок",
    },
    "1044": {
        "Type": STRING,
        "Head": "ОП.БАНК.АГЕНТА:",
        "Name": "операция банковского агента для банковских агентов",
    },
    "1045": {
        "Type": STRING,
        "Head": "",
        "Name": "операция банковского субагента для банковских агентов",
    },
    "1046": {"Type": STRING, "Head": "ОФД:", "Name": "ОФД"},
    "1047": {"Type": "", "Head": "", "Name": "параметр настройки"},
    "1048": {"Type": STRING, "Head": "", "Name": "наименование пользователя"},
    "1049": {"Type": "", "Head": "", "Name": "почтовый индекс"},
    "1050": {
        "Type": BYTE,
        "Head": "РЕСУРС ФН ИСЧЕРПАН",
        "Name": "признак исчерпания ресурса ФН",
    },
    "1051": {
        "Type": BYTE,
        "Head": "ЗАМЕНИТЬ ФН",
        "Name": "признак необходимости срочной замены ФН",
    },
    "1052": {
        "Type": BYTE,
        "Head": "ФН ПЕРЕПОЛНЕН",
        "Name": "признак переполнения памяти ФН",
    },
    "1053": {
        "Type": BYTE,
        "Head": "ОФД НЕ ОТВЕЧАЕТ",
        "Name": "признак превышения времени ожидания ответа ОФД",
    },
    "1054": {
        "Type": ENUM,
        "Head": "",
        "Name": "признак расчета",
        "Value1": "приход",
        "Value2": "возврат прихода",
        "Value3": "расход",
        "Value4": "возврат расхода",
    },
    "1055": {
        "Type": BITS,
        "Head": "СНО:",
        "Name": "применяемая система налогообложения для чека",
        "BIT0": "общая",
        "BIT1": "упрощенная доход",
        "BIT2": "упрощенная доход минус расход",
        "BIT3": "единый налог на вмененный доход",
        "BIT4": "единый сельскохозяйственный налог",
        "BIT5": "патентная система налогообложения",
        "BIT6": "упрощенная без представления налоговой декларации",
    },
    "1056": {"Type": BYTE, "Head": "ШФД", "Name": "признак шифрования"},
    "1057": {
        "Type": BITS,
        "Head": "",
        "Name": "применение платежными агентами (субагентами)",
        "BIT0": "банк. пл. агент",
        "BIT1": "банк. пл. субагент",
        "BIT2": "плат. агент",
        "BIT3": "плат. субагент",
        "BIT4": "поверенный",
        "BIT5": "комиссионер",
        "BIT6": "агент",
    },
    "1058": {
        "Type": STRING,
        "Head": "",
        "Name": "применение банковскими агентами (субагентами)",
    },
    "1059": {"Type": STLV, "Head": "", "Name": "наименование товара (реквизиты)"},
    "1060": {"Type": STRING, "Head": "САЙТ ФНС:", "Name": "сайт налогового органа"},
    "1061": {"Type": STRING, "Head": "", "Name": "сайт ОФД"},
    "1062": {
        "Type": BITS,
        "Head": "СНО:",
        "Name": "системы налогообложения для регистрации ",
        "BIT0": "ОСН",
        "BIT1": "УСН доход",
        "BIT2": "УСН доход - расход",
        "BIT3": "ЕНВД",
        "BIT4": "ЕСН",
        "BIT5": "Патент",
    },
    "1063": {"Type": STRING, "Head": "", "Name": "скидка (ставка)"},
    "1064": {"Type": STRING, "Head": "", "Name": "скидка (сумма)"},
    "1065": {"Type": STRING, "Head": "", "Name": "сокращенное наименование налога"},
    "1066": {"Type": STRING, "Head": "", "Name": "сообщение"},
    "1067": {"Type": STRING, "Head": "", "Name": "сообщение оператора для ККТ"},
    "1068": {"Type": STLV, "Head": "", "Name": "сообщение оператора для ФН"},
    "1069": {"Type": STRING, "Head": "", "Name": "сообщение оператору"},
    "1070": {"Type": STRING, "Head": "", "Name": "ставка налога"},
    "1071": {
        "Type": STRING,
        "Head": "",
        "Name": "сторно товара (реквизиты)",
        "Note": "запрещается применение реквизита «сторно» в кассовых чеках (БСО), "
        "где отсутствует хотя бы один реквизит «наименование товара»",
    },
    "1072": {"Type": STRING, "Head": "", "Name": "сумма налога"},
    "1073": {
        "Type": STRING,
        "Head": "ТЛФ.ПЛ.АГЕНТА:",
        "Name": "телефон банковского агента для банковских агентов",
    },
    "1074": {
        "Type": STRING,
        "Head": "ТЛФ.ОП.ПР.ПЛАТЕЖА:",
        "Name": "телефон платежного агента для платежных агентов",
    },
    "1075": {
        "Type": STRING,
        "Head": "ТЛФ.ОП.ПЕРЕВОДА:",
        "Name": "телефон оператора по переводу денежных средств для банковских агентов",
    },
    "1076": {"Type": STRING, "Head": "", "Name": "тип сообщения"},
    "1077": {"Type": FPD, "Head": "ФП:", "Name": "фискальный признак документа"},
    "1078": {"Type": BYTES, "Head": "", "Name": "фискальный признак оператора"},
    "1079": {"Type": COINS, "Head": "", "Name": "цена за единицу"},
    "1080": {"Type": STRING, "Head": "", "Name": "штриховой код"},
    "1081": {
        "Type": COINS,
        "Head": "ЭЛЕКТРОННЫМИ:",
        "Name": "форма расчета – электронными",
    },
    "1082": {"Type": STRING, "Head": "", "Name": "телефон банковского субагента"},
    "1083": {
        "Type": STRING,
        "Head": "",
        "Name": "телефон платежного субагента для банковских агентов",
    },
    "1084": {"Type": STLV, "Head": "", "Name": "дополнительный реквизит пользователя"},
    "1085": {
        "Type": STRING,
        "Head": "",
        "Name": "наименование дополнительного реквизита",
    },
    "1086": {"Type": STRING, "Head": "", "Name": "значение дополнительного реквизита"},
    "1088": {"Type": STRING, "Head": "", "Name": "приход наличными"},
    "1089": {"Type": STRING, "Head": "", "Name": "приход электронными"},
    "1090": {"Type": STRING, "Head": "", "Name": "возврат прихода наличными"},
    "1091": {"Type": STRING, "Head": "", "Name": "возврат прихода электронными"},
    "1092": {"Type": STRING, "Head": "", "Name": "расход наличными"},
    "1093": {"Type": STRING, "Head": "", "Name": "расход электронными"},
    "1094": {"Type": STRING, "Head": "", "Name": "возврат расхода наличными"},
    "1095": {"Type": STRING, "Head": "", "Name": "возврат расхода электронными"},
    "1096": {
        "Type": STRING,
        "Head": "",
        "Name": "номер корректируемого фискального документа",
    },
    "1097": {
        "Type": INT32,
        "Head": "НЕПЕРЕДАННЫХ ФД:",
        "Name": "количество непереданных ФД",
    },
    "1098": {
        "Type": UNIXTIME,
        "Head": "ФД НЕ ПЕРЕДАНЫ С:",
        "Name": "дата и время первого из непереданных ФД",
    },
    "1100": {"Type": STRING, "Head": "", "Name": "номер предписания"},
    "1101": {
        "Type": ENUM,
        "Head": "ИЗМ.СВЕД.О ККТ:",
        "Name": "код причины перерегистрации",
        "Value1": "Замена ФН",
        "Value2": "Замена ОФД",
        "Value3": "Изменение реквизитов",
        "Value4": "Изменение настроек ККТ",
    },
    "1102": {"Type": COINS, "Head": "СУММА НДС 20%", "Name": "НДС со ставкой 20%"},
    "1103": {"Type": COINS, "Head": "СУММА НДС 10%", "Name": "НДС со ставкой 10%"},
    "1104": {
        "Type": COINS,
        "Head": "СУММА НДС 0%",
        "Name": "НДС со ставкой 0% (сумма оборота по налогу)",
    },
    "1105": {
        "Type": COINS,
        "Head": "СУММА БЕЗ НДС:",
        "Name": "НДС не облагается (сумма оборота по налогу)",
    },
    "1106": {
        "Type": COINS,
        "Head": "СУММА НДС 20/120",
        "Name": "НДС с рассчитанной ставкой 20%",
    },
    "1107": {
        "Type": COINS,
        "Head": "СУММА НДС 10/110",
        "Name": "НДС с рассчитанной ставкой 10%",
    },
    "1108": {
        "Type": BYTE,
        "Head": "РАСЧЕТ ТОЛЬКО В ИНТЕРНЕТ",
        "Name": "признак расчетов в сети Интернет",
    },
    "1109": {
        "Type": BYTE,
        "Head": "ТОЛЬКО УСЛУГИ",
        "Name": "признак работы в сфере услуг",
    },
    "1110": {
        "Type": BYTE,
        "Head": "АС БСО",
        "Name": "применяется для формирования БСО",
    },
    "1111": {
        "Type": INT32,
        "Head": "ФД ЗА СМЕНУ",
        "Name": "количество фискальных документов за смену",
    },
    "1112": {"Type": STRING, "Head": "", "Name": "скидка/наценка"},
    "1113": {"Type": STRING, "Head": "", "Name": "наименование скидки"},
    "1114": {"Type": STRING, "Head": "", "Name": "наименование наценки"},
    "1115": {"Type": STRING, "Head": "", "Name": "адрес сайта для проверки ФП"},
    "1116": {
        "Type": INT32,
        "Head": "ПЕРВЫЙ НЕПЕРЕДАННЫЙ ФД:",
        "Name": "номер первого непереданного документа",
    },
    "1117": {"Type": STRING, "Head": "", "Name": "адрес отправителя"},
    "1118": {
        "Type": INT32,
        "Head": "ЧЕКОВ ЗА СМЕНУ:",
        "Name": "количество кассовых чеков за смену",
    },
    "1119": {"Type": STRING, "Head": "", "Name": "телефон оператора по приему платежей"},
    "1120": {"Type": STRING, "Head": "", "Name": "код справочника"},
    "1121": {"Type": STRING, "Head": "", "Name": "код классификации товара"},
    "1122": {"Type": STRING, "Head": "", "Name": "сведения о классификации товара"},
    "1123": {"Type": STRING, "Head": "", "Name": "код идентификации товара"},
    "1124": {"Type": STRING, "Head": "", "Name": "сведения об идентификации товара"},
    "1126": {
        "Type": BYTE,
        "Head": "ПРОВЕДЕНИЕ ЛОТЕРЕИ",
        "Name": "признак проведения лотереи",
    },
    "1129": {"Type": STLV, "Head": "", "Name": 'счётчики операций "приход"'},
    "1130": {"Type": STLV, "Head": "", "Name": 'счётчики операций "возврат прихода"'},
    "1131": {"Type": STLV, "Head": "", "Name": 'счётчики операций "расход"'},
    "1132": {"Type": STLV, "Head": "", "Name": 'счётчики операций "возврат расхода"'},
    "1133": {"Type": STLV, "Head": "", "Name": "счётчики операций по чекам коррекции"},
    "1134": {
        "Type": INT32,
        "Head": "",
        "Name": "количество чеков со всеми признаками расчетов",
    },
    "1135": {
        "Type": INT32,
        "Head": "",
        "Name": "количество чеков по признаку расчетов",
    },
    "1136": {"Type": COINS, "Head": "", "Name": "итоговая сумма в чеках наличными"},
    "1138": {"Type": COINS, "Head": "", "Name": "итоговая сумма в чеках электронными"},
    "1139": {"Type": COINS, "Head": "", "Name": "сумма НДС по ставке 20%"},
    "1140": {"Type": COINS, "Head": "", "Name": "сумма НДС по ставке 10%"},
    "1141": {"Type": COINS, "Head": "", "Name": "сумма НДС по ставке 20/120"},
    "1142": {"Type": COINS, "Head": "", "Name": "сумма НДС по ставке 10/110"},
    "1143": {"Type": COINS, "Head": "", "Name": "сумма НДС по ставке 0%"},
    "1144": {"Type": INT32, "Head": "", "Name": "количество чеков коррекции"},
    "1145": {"Type": STLV, "Head": "", "Name": 'счётчики коррекций "приход"'},
    "1146": {"Type": STLV, "Head": "", "Name": 'счётчики коррекций "расход"'},
    "1148": {
        "Type": INT32,
        "Head": "",
        "Name": "количество самостоятельных корректировок",
    },
    "1149": {
        "Type": INT32,
        "Head": "",
        "Name": "количество корректировок по предписанию",
    },
    "1151": {"Type": COINS, "Head": "", "Name": "сумма коррекций с НДС по ставке 20%"},
    "1152": {"Type": COINS, "Head": "", "Name": "сумма коррекций с НДС по ставке 10%"},
    "1153": {
        "Type": COINS,
        "Head": "",
        "Name": "сумма коррекций с НДС по ставке 20/120",
    },
    "1154": {
        "Type": COINS,
        "Head": "",
        "Name": "сумма коррекций с НДС по ставке 10/110",
    },
    "1155": {"Type": COINS, "Head": "", "Name": "сумма коррекций с НДС по ставке 0%"},
    "1157": {"Type": STLV, "Head": "", "Name": "счётчики итогов ФН"},
    "1158": {"Type": STLV, "Head": "", "Name": "счётчики итогов непереданных ФД"},
    "1162": {"Type": BYTES, "Head": "", "Name": "код товарной номенклатуры"},
    "1163": {"Type": STLV, "Head": "", "Name": "код товара"},
    "1171": {"Type": STRING, "Head": "", "Name": "телефон поставщика"},
    "1173": {
        "Type": ENUM,
        "Head": "",
        "Name": "тип коррекции",
        "Value0": "самостоятельно",
        "Value1": "по предписанию",
    },
    "1174": {"Type": STLV, "Head": "", "Name": "основание для коррекции"},
    "1177": {
        "Type": STRING,
        "Head": "",
        "Name": "наименование основания для коррекции",
    },
    "1178": {
        "Type": UNIXTIME,
        "Head": "",
        "Name": "дата документа основания для коррекции",
    },
    "1179": {
        "Type": INT32,
        "Head": "",
        "Name": "номер документа основания для коррекции",
    },
    "1183": {"Type": COINS, "Head": "", "Name": "сумма расчетов без НДС"},
    "1184": {"Type": COINS, "Head": "", "Name": "сумма коррекций без НДС"},
    "1187": {"Type": STRING, "Head": "", "Name": "место расчетов"},
    "1188": {"Type": STRING, "Head": "", "Name": "версия ККТ"},
    "1189": {
        "Type": ENUM,
        "Head": "",
        "Name": "версия ФФД ККТ",
        "Value1": "1.0",
        "Value2": "1.05",
        "Value3": "1.1",
        "Value4": "1.2",
    },
    "1190": {
        "Type": ENUM,
        "Head": "",
        "Name": "версия ФФД ФН",
        "Value1": "1.0",
        "Value2": "1.05",
        "Value3": "1.1",
        "Value4": "1.2",
    },
    "1191": {
        "Type": STRING,
        "Head": "",
        "Name": "дополнительный реквизит предмета расчета",
    },
    "1192": {"Type": STRING, "Head": "", "Name": "дополнительный реквизит чека"},
    "1193": {"Type": BYTE, "Head": "", "Name": "признак проведения азартных игр"},
    "1194": {"Type": STLV, "Head": "", "Name": "счетчики итогов смены"},
    "1197": {"Type": STRING, "Head": "", "Name": "единица измерения предмета расчета"},
    "1198": {
        "Type": COINS,
        "Head": "",
        "Name": "размер НДС за единицу предмета расчета",
    },
    "1199": {
        "Type": ENUM,
        "Head": "",
        "Name": "ставка НДС",
        "Value1": "ставка НДС 20%",
        "Value2": "ставка НДС 10%",
        "Value3": "ставка НДС расч. 20/120",
        "Value4": "ставка НДС расч. 10/110",
        "Value5": "ставка НДС 0%",
        "Value6": "НДС не облагается",
    },
    "1200": {
        "Type": COINS,
        "Head": "",
        "Name": "сумма НДС за предмет расчета",
    },
    "1201": {
        "Type": COINS,
        "Head": "",
        "Name": "общая итоговая сумма в чеках",
    },
    "1203": {
        "Type": STRING,
        "Head": "",
        "Name": "ИНН кассира",
    },
    "1205": {
        "Type": INT32,
        "Head": "",
        "Name": "коды причин изменения сведений о ККТ",
    },
    "1206": {
        "Type": BITS,
        "Head": "",
        "Name": "сообщение оператора",
        "BIT1": "ошибка ФЛК",
        "BIT6": "требуется настроить ККТ",
        "BIT7": "ОФД аннулирован",
    },
    "1207": {
        "Type": BYTE,
        "Head": "",
        "Name": "признак торговли подакцизными товарами",
    },
    "1208": {
        "Type": STRING,
        "Head": "",
        "Name": "сайт чеков",
    },
    "1209": {
        "Type": ENUM,
        "Head": "",
        "Name": "версия ФФД",
        "Value1": "1.0",
        "Value2": "1.05",
        "Value3": "1.1",
        "Value4": "1.2",
    },
    "1212": {
        "Type": ENUM,
        "Head": "",
        "Name": "признак предмета расчета",
        "Value1": "товар",
        "Value2": "подакцизный товар",
        "Value3": "работа",
        "Value4": "услуга",
        "Value5": "ставка азартной игры",
        "Value6": "выигрыш азартной игры",
        "Value7": "лотерейный билет",
        "Value8": "выигрыш лотереи",
        "Value9": "предоставление РИД",
        "Value10": "платеж (выплата)",
        "Value11": "агентское вознаграждение",
        "Value12": "составной предмет расчета",
        "Value13": "иной предмет расчета",
        "Value14": "имущественное право",
        "Value15": "внереализационный доход",
        "Value16": "стаховой взнос",
        "Value17": "торговый взнос",
        "Value18": "курортный сбор",
        "Value20": "",
        "Value21": "",
        "Value22": "",
        "Value23": "",
        "Value24": "",
        "Value25": "",
        "Value26": "",
        "Value27": "",
        "Value28": "",
        "Value29": "",
        "Value30": "",
        "Value31": "",
    },
    "1213": {"Type": INT16, "Head": "", "Name": "ресурс ключей ФП"},
    "1214": {
        "Type": ENUM,
        "Head": "",
        "Name": "признак способа расчета",
        "Value1": "предоплата 100%",
        "Value2": "предоплата",
        "Value3": "аванс",
        "Value4": "полный расчет",
        "Value5": "частичный расчет и кредит",
        "Value6": "передача в кредит",
        "Value7": "оплата кредита",
    },
    "1215": {
        "Type": COINS,
        "Head": "",
        "Name": "сумма по чеку предоплатой (зачетом аванса)",
    },
    "1216": {
        "Type": COINS,
        "Head": "",
        "Name": "сумма по чеку постоплатой (в кредит)",
    },
    "1217": {
        "Type": COINS,
        "Head": "",
        "Name": "сумма по чеку встречным предоставлением",
    },
    "1218": {
        "Type": COINS,
        "Head": "",
        "Name": "итоговая сумма в чеках предоплатами (авансами)",
    },
    "1219": {
        "Type": COINS,
        "Head": "",
        "Name": "итоговая сумма в чеках постоплатами (кредитами)",
    },
    "1220": {
        "Type": COINS,
        "Head": "",
        "Name": "итоговая сумма в чеках встречными предоставлениями",
    },
    "1221": {
        "Type": BYTE,
        "Head": "",
        "Name": "признак установки принтера в автомате",
    },
    "1222": {
        "Type": BITS,
        "Head": "",
        "Name": "признак агента по предмету расчета",
        "BIT0": "банк. пл. агент",
        "BIT1": "банк. пл. субагент",
        "BIT2": "плат. агент",
        "BIT3": "плат. субагент",
        "BIT4": "поверенный",
        "BIT5": "комиссионер",
        "BIT6": "агент",
    },
    "1223": {
        "Type": STLV,
        "Head": "",
        "Name": "данные агента",
    },
    "1224": {
        "Type": STLV,
        "Head": "",
        "Name": "данные поставщика",
    },
    "1225": {
        "Type": STRING,
        "Head": "",
        "Name": "наименование поставщика",
    },
    "1226": {"Type": STRING, "Head": "", "Name": "ИНН поставщика"},
    "1227": {"Type": STRING, "Head": "", "Name": "покупатель (клиент)"},
    "1228": {"Type": STRING, "Head": "", "Name": "ИНН покупателя (клиента)"},
    "1229": {"Type": COINS, "Head": "", "Name": "акциз"},
    "1230": {"Type": STRING, "Head": "", "Name": "код страны происхождения товара"},
    "1231": {"Type": STRING, "Head": "", "Name": "номер таможенной декларации"},
    "1232": {
        "Type": STLV,
        "Head": "",
        "Name": 'счетчики по признаку "возврат прихода"',
    },
    "1233": {
        "Type": STLV,
        "Head": "",
        "Name": 'счетчики по признаку "возврат расхода"',
    },
    "1257": {
        "Type": BYTE,
        "Head": "",
        "Name": "признак осуществления ломбардной деятельности",
    },
    "1258": {
        "Type": BYTE,
        "Head": "",
        "Name": "признак осуществления страховой деятельности",
    },
    "1260": {
        "Type": STLV,
        "Head": "отраслевой реквизит предмета расчета",
        "Name": "отраслевой реквизит предмета расчета",
    },
    "1290": {
        "Type": BITS,
        "Head": "",
        "Name": "признаки условий применения ККТ",
        "BIT1": "ПРИНТЕР В АВТОМАТЕ",
        "BIT2": "АС БСО",
        "BIT5": "ККТ ДЛЯ ИНТЕРНЕТ",
        "BIT6": "ПОДАКЦИЗНЫЕ ТОВАРЫ",
        "BIT8": "ТМТ",
        "BIT9": "ККТ ДЛЯ УСЛУГ",
        "BIT10": "ПРОВЕДЕНИЕ АЗАРТНОЙ ИГРЫ",
        "BIT11": "ПРОВЕДЕНИЕ ЛОТЕРЕИ",
        "BIT12": "ЛОМБАРД",
        "BIT13": "СТРАХОВАНИЕ",
    },
    "1300": {"Type": STRING, "Head": "", "Name": "нераспознанный код товара"},
    "1301": {"Type": STRING, "Head": "", "Name": "КТ EAN-8"},
    "1302": {"Type": STRING, "Head": "", "Name": "КТ EAN-13"},
    "1303": {"Type": STRING, "Head": "", "Name": "КТ ITF-14"},
    "1304": {"Type": STRING, "Head": "", "Name": "КТ GS1.0"},
    "1305": {"Type": STRING, "Head": "", "Name": "КТ GS1.М"},
    "1306": {"Type": STRING, "Head": "", "Name": "КТ КМК"},
    "1307": {"Type": STRING, "Head": "", "Name": "КТ МИ"},
    "1308": {"Type": STRING, "Head": "", "Name": "КТ ЕГАИС-2.0"},
    "1309": {"Type": STRING, "Head": "", "Name": "КТ ЕГАИС-3.0"},
    "1320": {"Type": STRING, "Head": "", "Name": "КТ Ф.1"},
    "1321": {"Type": STRING, "Head": "", "Name": "КТ Ф.2"},
    "1322": {"Type": STRING, "Head": "", "Name": "КТ Ф.3"},
    "1323": {"Type": STRING, "Head": "", "Name": "КТ Ф.4"},
    "1324": {"Type": STRING, "Head": "", "Name": "КТ Ф.5"},
    "1325": {"Type": STRING, "Head": "", "Name": "КТ Ф.6"},
    "2000": {"Type": STRING, "Head": "", "Name": "код маркировки"},
    "2007": {"Type": COINS, "Head": "", "Name": "данные о маркированном товаре"},
    "2100": {"Type": BYTE, "Head": "", "Name": "тип кода маркировки"},
    "2101": {"Type": STRING, "Head": "", "Name": "код идентификации товара"},
    "2102": {
        "Type": BYTE,
        "Head": "",
        "Name": "режим обработки кода маркировки",
    },
    "2103": {
        "Type": BYTE,
        "Head": "",
        "Name": "признак торговли маркированными товарами",
    },
    "2106": {
        "Type": BYTE,
        "Head": "",
        "Name": "результат проверки сведений о товаре",
    },
    "2107": {
        "Type": BYTE,
        "Head": "",
        "Name": "результаты проверки маркированных товаров",
    },
    "2108": {
        "Type": BYTE,
        "Head": "",
        "Name": "ответ ОИСМ о статусе товара",
    },
    "2109": {
        "Type": BYTE,
        "Head": "",
        "Name": "мера количества предмета расчета",
    },
    "2110": {
        "Type": BYTE,
        "Head": "",
        "Name": "присвоенный статус товара",
    },
    "2115": {
        "Type": STRING,
        "Head": "",
        "Name": "код идентификации вида товара",
    },
    "2117": {
        "Type": COINS,
        "Head": "",
        "Name": "отраслевой реквизит чека",
    },
    "65001": {
        "Type": STLV,
        "Head": "",
        "Name": "ФДн в автономном режиме для ФФД 1.0 и 1.05",
    },
    "65002": {
        "Type": STLV,
        "Head": "",
        "Name": "ФДн и подтверждение для ФФД 1.0 и 1.05",
    },
    "65011": {
        "Type": STLV,
        "Head": "",
        "Name": "ФДн в автономном режиме для ФФД ФФД 1.1",
    },
    "65012": {
        "Type": STLV,
        "Head": "",
        "Name": "ФДн и подтверждение для ФФД 1.1",
    },
    "fiscalDocumentType": {
        "Type": STRING,
        "Head": "",
        "Name": "Тип документа",
    },
    "qr": {
        "Type": STRING,
        "Head": "",
        "Name": "QR код чека 1196",
    },
    "short": {
        "Type": STRING,
        "Head": "",
        "Name": "Признак того, что документ считался из архива и содержит неполный набор данных",
    },
}



import json
import logging
import threading
import time

from ofdcomparer.dto_error_descriptions import get_dto_error_code_description
from ofdcomparer.libfptr10 import IFptr


class DTO10Helper:
    def __init__(self, fptr=None):
        self.fptr = fptr

        # Информация о ФН
        self.fnWarnings = None
        self.fnVersion = None
        self.fnValidityDate = None
        self.fnSerial = None
        self.fnRegistrationsRemaining = None
        self.fnNumberOfRegistrations = None
        self.fnLivePhase = None
        self.fnFfdVersion = None
        self.fnContainsKeysUpdaterServerUri = None
        self.FfdVersion = None
        self.fnExecution = None

        # Информация о регистрации
        self.device = None
        self.autoMode = None
        self.bso = None
        self.defaultTaxationType = None
        self.encryption = None
        self.excise = None
        self.ffdVersion = None
        self.fnsUrl = None
        self.gambling = None
        self.internet = None
        self.lottery = None
        self.machineInstallation = None
        self.machineNumber = None
        self.ofdChannel = None
        self.offlineMode = None
        self.paymentsAddress = None
        self.registrationNumber = None
        self.service = None

        # ofd
        self.ofd = None
        self.dns = None
        self.host = None
        self.name = None
        self.port = None
        self.vatin = None

        # organization
        self.organization = None
        self.address = None
        self.agents = None
        self.email = None
        self.name = None
        self.taxationTypes = None
        self.vatin = None

        if fptr is None:
            while True:
                try:
                    self.fptr = IFptr()
                except Exception:
                    raise Exception("Error creating DTO 10")
                if self.fptr.interface.value is not None:
                    break

        self.dto10_version = self.fptr.version().decode("cp866")

        self.ErrorCode = None
        self.ErrorDescription = None
        self.JsonAnswer = None

        if fptr is None:
            self.fptr.setSingleSetting(
                self.fptr.LIBFPTR_SETTING_MODEL, str(self.fptr.LIBFPTR_MODEL_ATOL_AUTO)
            )
        self.RNM = self.get_rnm_number()
        self.FN = self.get_fn_number()
        self.get_registration_data()
        self.get_fn_data()

    def get_information_and_status(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_DATA_TYPE, self.fptr.LIBFPTR_DT_STATUS
        )
        self.fptr.queryData()
        self.request_error_code()
        serial_number = self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_SERIAL_NUMBER)
        return serial_number

    def open(self):
        self.fptr.open()
        return self.request_error_code()

    def close(self):
        self.fptr.close()
        return self.request_error_code()

    def request_error_code(self, flag_all_kkt_settings=False):
        self.ErrorCode = self.fptr.errorCode()
        self.ErrorDescription = self.fptr.errorDescription()
        pytest_dto_error_description = get_dto_error_code_description(self.ErrorCode)
        if self.ErrorDescription != pytest_dto_error_description:
            logging.error(f"Код ошибки ДТО: {self.ErrorCode}")
            logging.error(
                f"ДТО [{self.ErrorDescription}] <-> "
                f"[{pytest_dto_error_description}] const.dto_error_descriptions"
            )
            logging.error(
                "НЕОБХОДИМО АКТУАЛИЗИРОВАТЬ СЛОВАРЬ ALL_DTO10_ERROR_DESCRIPTIONS"
            )
        if self.ErrorCode != self.fptr.LIBFPTR_OK:
            if (
                    flag_all_kkt_settings
                    and self.ErrorCode == self.fptr.LIBFPTR_ERROR_NOT_SUPPORTED
            ):
                pass
            else:
                logging.error(
                    "[ДТО]: Код ошибки: {} [{}]".format(
                        self.ErrorCode, self.ErrorDescription
                    )
                )
        return self.ErrorCode

    def get_last_fiscal_document_number(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_FN_DATA_TYPE, self.fptr.LIBFPTR_FNDT_LAST_DOCUMENT
        )
        self.fptr.fnQueryData()
        self.request_error_code()
        return int(self.fptr.getParamInt(self.fptr.LIBFPTR_PARAM_DOCUMENT_NUMBER))

    def get_last_fd_number(self):
        logging.debug("get_last_fd_number()")
        try:
            json_task = {"type": "getFnStatus"}
            fn_status = self.execute_json(json_task)
            logging.debug("СТАТУС ФН: %s", fn_status)
            last_fd_number = fn_status["fnStatus"]["fiscalDocumentNumber"]
            logging.debug(f"get_last_fd_number() > {last_fd_number}")
            logging.info("ФД: %s", last_fd_number)
            return last_fd_number
        except Exception as e:
            logging.error("get_last_fd_number error: {}".format(e))
            logging.debug("get_last_fd_number() > None")
            return None

    def get_fd_from_fn(self, fd: int):
        """
        Извлечение ФД из ФН
        """
        logging.debug(f"get_fd_from_fn() < {fd}")
        if fd is None:
            logging.debug(f"get_fd_from_fn() > None")
            return None
        # JSON задание для чтения документа из ФН

        json_task = {
            "type": "getFnDocument",
            "fiscalDocumentNumber": int(fd),
            "withRawData": False,
        }
        fd_fn = self.execute_json(json_task)
        if fd_fn is None:
            return None
        if isinstance(fd_fn, dict):
            fd_fn = fd_fn["documentTLV"]
            return fd_fn
        return None

    def connect_to_kkt_by_usb(self, usb_device_path="auto"):
        logging.debug("Устанавливаем соединение с ККТ по USB...")
        self.set_connection_settings_to_usb(usb_device_path)
        if self.open() != self.fptr.LIBFPTR_OK:
            logging.error("Не удалось установить соединение с ККТ")
            return False
        if not self.is_connection_type_usb():
            return False
        logging.info("Соединение с ККТ установлено по USB")
        return True

    def connect_to_kkt_by_ethernet(self, ip="127.0.0.1", port="5555"):
        logging.debug(f"Устанавливаем соединение по ip {ip}:{port}")
        self.set_connection_settings_to_ethernet(ip, port)
        err = self.open()
        if err != self.fptr.LIBFPTR_OK:
            logging.info(f"Ошибка - не удалось установить соединение с ККТ ({err})")
            return False
        if not self.is_connection_type_ethernet():
            return False
        logging.info(
            f"Соединение с ККТ установлено по Ethernet. ip=[{ip}] port=[{port}]"
        )
        return True

    def set_connection_settings_to_ethernet(self, ip, port):
        self.fptr.setSingleSetting(
            self.fptr.LIBFPTR_SETTING_PORT, str(self.fptr.LIBFPTR_PORT_TCPIP)
        )
        self.fptr.setSingleSetting(self.fptr.LIBFPTR_SETTING_IPADDRESS, str(ip))
        self.fptr.setSingleSetting(self.fptr.LIBFPTR_SETTING_IPPORT, str(port))
        self.fptr.applySingleSettings()
        return self.request_error_code()

    def is_connection_type_ethernet(self):
        if not self.fptr.isOpened():
            return False
        connection_type = self.fptr.getSingleSetting(self.fptr.LIBFPTR_SETTING_PORT)
        if not int(connection_type) == self.fptr.LIBFPTR_PORT_TCPIP:
            logging.error(
                f"Способ связи с ККТ [{connection_type}], а должен быть [Ethernet]"
            )
            return False
        return True

    def set_connection_settings_to_usb(self, usb_device_path):
        self.fptr.setSingleSetting(
            self.fptr.LIBFPTR_SETTING_PORT, str(self.fptr.LIBFPTR_PORT_USB)
        )
        self.fptr.setSingleSetting(
            self.fptr.LIBFPTR_SETTING_USB_DEVICE_PATH, str(usb_device_path)
        )
        self.fptr.applySingleSettings()
        return self.request_error_code()

    def is_connection_type_usb(self):
        if not self.fptr.isOpened():
            return False
        connection_type = self.fptr.getSingleSetting(self.fptr.LIBFPTR_SETTING_PORT)
        self.request_error_code()
        if not int(connection_type) == self.fptr.LIBFPTR_PORT_USB:
            logging.error(
                "Способ связи с ККТ [{}], а должен быть [USB]".format(connection_type)
            )
            return False
        return True

    def connect_to_kkt_by_rs(self, com="COM1", baudrate=115200):
        logging.debug("Устанавливаем соединение по RS")
        if self.set_connection_settings_to_rs(com, baudrate) != self.fptr.LIBFPTR_OK:
            logging.debug("Не удалось установить настройки драйвера для подключения к ККТ")
            return False
        if self.open() != self.fptr.LIBFPTR_OK:
            logging.debug("Не удалось установить соединение с ККТ")
            return False
        if not self.is_connection_type_rs():
            return False
        logging.debug(
            "Соединение с ККТ установлено по RS. com=[{}] baudrate=[{}]".format(
                com, baudrate
            )
        )
        return True

    def set_connection_settings_to_rs(self, com, baudrate):
        self.fptr.setSingleSetting(
            self.fptr.LIBFPTR_SETTING_MODEL, str(self.fptr.LIBFPTR_MODEL_ATOL_AUTO)
        )
        self.fptr.setSingleSetting(
            self.fptr.LIBFPTR_SETTING_PORT, str(self.fptr.LIBFPTR_PORT_COM)
        )
        self.fptr.setSingleSetting(self.fptr.LIBFPTR_SETTING_COM_FILE, com)
        self.fptr.setSingleSetting(self.fptr.LIBFPTR_SETTING_BAUDRATE, str(baudrate))
        self.fptr.applySingleSettings()
        return self.request_error_code()

    def is_connection_type_rs(self):
        if not self.fptr.isOpened():
            return False
        connection_type = self.fptr.getSingleSetting(self.fptr.LIBFPTR_SETTING_PORT)
        self.request_error_code()
        if not int(connection_type) == self.fptr.LIBFPTR_PORT_COM:
            logging.error(
                "Способ связи с ККТ [{}], а должен быть [RS]".format(connection_type)
            )
            return False
        return True

    def get_model_name(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_DATA_TYPE, self.fptr.LIBFPTR_DT_MODEL_INFO
        )
        self.fptr.queryData()
        model_name = str(self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_MODEL_NAME))
        return model_name

    def set_no_print_flag(self, no_print):
        self.fptr.setParam(self.fptr.LIBFPTR_PARAM_REPORT_ELECTRONICALLY, no_print)
        return True

    def reset_settings(self):
        self.fptr.resetSettings()
        return self.request_error_code()

    def get_fatal_errors_status(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_DATA_TYPE, self.fptr.LIBFPTR_DT_FATAL_STATUS
        )
        self.fptr.queryData()

        self.request_error_code()

        no_serial_number = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_NO_SERIAL_NUMBER
        )
        rtc_fault = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_RTC_FAULT)
        settings_fault = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_SETTINGS_FAULT)
        counter_fault = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_COUNTERS_FAULT)
        user_memory_fault = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_USER_MEMORY_FAULT
        )
        service_counters_fault = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_SERVICE_COUNTERS_FAULT
        )
        attributes_fault = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_ATTRIBUTES_FAULT
        )
        fn_fault = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_FN_FAULT)
        invalid_fn = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_INVALID_FN)
        hard_fault = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_HARD_FAULT)
        memory_manager_fault = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_MEMORY_MANAGER_FAULT
        )
        script_fault = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_SCRIPTS_FAULT)
        wait_for_reboot = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_WAIT_FOR_REBOOT
        )
        universal_counters_fault = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_UNIVERSAL_COUNTERS_FAULT
        )
        commodities_table_fault = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_COMMODITIES_TABLE_FAULT
        )

        return (
            no_serial_number,
            rtc_fault,
            settings_fault,
            counter_fault,
            user_memory_fault,
            service_counters_fault,
            attributes_fault,
            fn_fault,
            invalid_fn,
            hard_fault,
            memory_manager_fault,
            script_fault,
            wait_for_reboot,
            universal_counters_fault,
            commodities_table_fault,
        )

    def reboot_kkt(self):
        self.fptr.deviceReboot()
        return self.request_error_code()

    def set_setting_id(self, _id, _value):
        self.fptr.setParam(self.fptr.LIBFPTR_PARAM_SETTING_ID, _id)
        self.fptr.setParam(self.fptr.LIBFPTR_PARAM_SETTING_VALUE, _value)
        self.fptr.writeDeviceSetting()
        self.request_error_code()
        if self.ErrorCode != self.fptr.LIBFPTR_OK:
            return self.ErrorCode
        self.fptr.commitSettings()
        return self.request_error_code()

    def get_setting_id(self, _id):
        self.fptr.setParam(self.fptr.LIBFPTR_PARAM_SETTING_ID, _id)
        self.fptr.readDeviceSetting()
        self.request_error_code()
        return self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_SETTING_VALUE)

    def print_string(
            self,
            text=None,
            alignment=None,
            wrap=None,
            font=None,
            double_width=None,
            double_height=None,
            linespacing=None,
            brightness=None,
            store_in_journal=None,
    ):
        self.fptr.beginNonfiscalDocument()
        if text is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_TEXT, text)
        if alignment is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_ALIGNMENT, alignment)
        if wrap is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_TEXT_WRAP, wrap)
        if font is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_FONT, font)
        if double_width is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_FONT_DOUBLE_WIDTH, double_width)
        if double_height is not None:
            self.fptr.setParam(
                self.fptr.LIBFPTR_PARAM_FONT_DOUBLE_HEIGHT, double_height
            )
        if linespacing is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_LINESPACING, linespacing)
        if brightness is not None:
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_BRIGHTNESS, brightness)
        if store_in_journal is not None:
            self.fptr.setParam(
                self.fptr.LIBFPTR_PARAM_STORE_IN_JOURNAL, store_in_journal
            )
        self.fptr.printText()
        self.fptr.endNonfiscalDocument()
        return self.request_error_code()

    def print_custom_text(self, text_tuple: tuple):
        """
        Метод печати нефискального текста на ЧЛ
        """
        self.fptr.open()
        self.fptr.beginNonfiscalDocument()
        for text in text_tuple:
            self.fptr.setParam(IFptr.LIBFPTR_PARAM_TEXT, text)
            self.fptr.setParam(IFptr.LIBFPTR_PARAM_ALIGNMENT, 0)
            self.fptr.setParam(IFptr.LIBFPTR_PARAM_TEXT_WRAP, 0)
            self.fptr.setParam(IFptr.LIBFPTR_PARAM_FONT, 0)
            self.fptr.setParam(IFptr.LIBFPTR_PARAM_FONT_DOUBLE_HEIGHT, False)
            self.fptr.setParam(IFptr.LIBFPTR_PARAM_FONT_DOUBLE_WIDTH, False)
            self.fptr.setParam(IFptr.LIBFPTR_PARAM_FORMAT_TEXT, False)
            self.fptr.setParam(IFptr.LIBFPTR_PARAM_LINESPACING, 0)
            self.fptr.setParam(IFptr.LIBFPTR_PARAM_BRIGHTNESS, 0)
            self.fptr.setParam(IFptr.LIBFPTR_PARAM_STORE_IN_JOURNAL, True)
            self.fptr.printText()
        self.fptr.setParam(IFptr.LIBFPTR_PARAM_PRINT_FOOTER, False)
        self.fptr.endNonfiscalDocument()
        self.fptr.close()

    def get_fn_info(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_FN_DATA_TYPE, self.fptr.LIBFPTR_FNDT_FN_INFO
        )
        self.fptr.fnQueryData()
        self.request_error_code()

        serial = self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_SERIAL_NUMBER)
        version = self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_FN_VERSION)
        type = self.fptr.getParamInt(self.fptr.LIBFPTR_PARAM_FN_TYPE)
        state = self.fptr.getParamInt(self.fptr.LIBFPTR_PARAM_FN_STATE)
        flags = self.fptr.getParamInt(self.fptr.LIBFPTR_PARAM_FN_FLAGS)

        need_replacement = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_FN_NEED_REPLACEMENT
        )
        exhausted = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_FN_RESOURCE_EXHAUSTED
        )
        memory_overflow = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_FN_MEMORY_OVERFLOW
        )
        ofd_timeout = self.fptr.getParamBool(self.fptr.LIBFPTR_PARAM_FN_OFD_TIMEOUT)
        critical_error = self.fptr.getParamBool(
            self.fptr.LIBFPTR_PARAM_FN_CRITICAL_ERROR
        )

        return (
            serial,
            version,
            type,
            state,
            flags,
            need_replacement,
            exhausted,
            memory_overflow,
            ofd_timeout,
            critical_error,
        )

    def init_mgm(self):
        self.fptr.initMgm()
        return self.request_error_code()

    def get_last_doc_with_template(self, tempalate, variable_settings=None):
        last_fn_doc_number = self.get_last_fiscal_document_number()
        last_fn_doc_str = None
        for _ in range(3):
            request_last_fn_doc = (
                    '{"type": "getFnDocument", "fiscalDocumentNumber": %i}'
                    % last_fn_doc_number
            )
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_JSON_DATA, request_last_fn_doc)
            self.fptr.processJson()
            last_fn_doc_str = self.fptr.getParamString(
                self.fptr.LIBFPTR_PARAM_JSON_DATA
            )
            if last_fn_doc_str != "":
                break
        last_fn_doc_dic = json.loads(str(last_fn_doc_str))
        last_fn_doc_dic = last_fn_doc_dic["documentTLV"]
        logging.info(
            "\nФД %s, JSON из ФН: %s"
            % (
                str(last_fn_doc_number),
                json.dumps(
                    last_fn_doc_dic, sort_keys=True, indent=4, ensure_ascii=False
                ),
            )
        )

        return last_fn_doc_dic

    def execute_json(self, json_task, timeout=30):
        logging.debug(f"execute_json() < json_task {json_task}, timeout {timeout}")
        try:
            self.fptr.open()
            self.fptr.setParam(self.fptr.LIBFPTR_PARAM_JSON_DATA, json.dumps(json_task))
            self.fptr.processJson()
            response_raw = self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_JSON_DATA)
            if response_raw is not None:
                try:
                    response = json.loads(response_raw)
                    logging.debug(f"execute_json() > {response}")
                    return response
                except json.decoder.JSONDecodeError as e:
                    logging.error(
                        "[ERROR] error with json.loads from response: {}".format(e)
                    )
        except ValueError as e:
            logging.error("Exception in execute_json(): {}".format(e))
        finally:
            self.fptr.close()  # Закрытие ДТО
        logging.debug(f"execute_json() > None")
        return None

    def get_last_fd_number(self):
        logging.debug("get_last_fd_number()")
        try:
            json_task = {"type": "getFnStatus"}
            fn_status = self.execute_json(json_task)
            logging.debug("СТАТУС ФН: %s", fn_status)
            last_fd_number = fn_status["fnStatus"]["fiscalDocumentNumber"]
            logging.debug(f"get_last_fd_number() > {last_fd_number}")
            logging.info("ФД: %s", last_fd_number)
            return last_fd_number
        except Exception as e:
            logging.error("get_last_fd_number error: {}".format(e))
            logging.debug("get_last_fd_number() > None")
            return None

    def get_fn_number(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_DATA_TYPE, self.fptr.LIBFPTR_DT_CACHE_REQUISITES
        )
        self.fptr.queryData()

        serialNumber = self.fptr.getParamString(
            self.fptr.LIBFPTR_PARAM_FN_SERIAL_NUMBER
        )
        return serialNumber

    def get_rnm_number(self):
        self.fptr.setParam(
            self.fptr.LIBFPTR_PARAM_DATA_TYPE, self.fptr.LIBFPTR_DT_CACHE_REQUISITES
        )
        self.fptr.queryData()

        ecrRegNumber = self.fptr.getParamString(
            self.fptr.LIBFPTR_PARAM_ECR_REGISTRATION_NUMBER
        )

        return ecrRegNumber

    def get_not_sent_fd_qty(self):
        logging.debug(f"get_not_sent_fd_qty()")
        task = {"type": "ofdExchangeStatus"}
        ofdExchangeStatus = self.execute_json(task)
        if ofdExchangeStatus is not None:
            not_sent_fd_qty = ofdExchangeStatus["status"]["notSentCount"]
            logging.debug(f"get_not_sent_fd_qty() %s", not_sent_fd_qty)
            return not_sent_fd_qty
        logging.debug(f"get_not_sent_fd_qty() None")
        return None

    def wait_for_sent_all_fd(self, timeout: int = 600):
        logging.debug(f"wait_for_sent_all_fd() timeout %s", timeout)
        start_time = time.time()
        while self.get_not_sent_fd_qty() != 0:
            if time.time() - start_time > timeout:
                logging.debug(f"get_not_sent_fd_qty() False")
                return False
            time.sleep(3)
        logging.debug(f"get_not_sent_fd_qty() True")
        return True

    def process_json(self, json_task, answer_indent=4, flag_all_kkt_settings=False):
        self.fptr.setParam(self.fptr.LIBFPTR_PARAM_JSON_DATA, json_task)
        self.fptr.processJson()
        error = self.request_error_code(flag_all_kkt_settings=flag_all_kkt_settings)
        json_str_answer = self.fptr.getParamString(self.fptr.LIBFPTR_PARAM_JSON_DATA)
        if json_str_answer != "":
            self.JsonAnswer = None
            try:
                self.JsonAnswer = json.dumps(
                    json.loads(json_str_answer),
                    ensure_ascii=False,
                    indent=answer_indent,
                )
            except ValueError:
                logging.error("Не удалось считать структуру из json-ответа")
        return error

    def execute_mass_json(self, json_task, quantity, timeout=30):
        logging.debug(f"execute_json() < json_task {json_task}, timeout {timeout}")
        logging.debug(f"execute_json() < json_task {json_task}, timeout {timeout}")
        try:
            if not self.is_connected():
                if not self.wait_to_connect(timeout=timeout):
                    return False
            self.fptr.open()
            count = 0
            while count < quantity:
                self.fptr.setParam(
                    self.fptr.LIBFPTR_PARAM_JSON_DATA, json.dumps(json_task)
                )
                self.fptr.processJson()
                logging.debug(count)
                count += 1
        except:
            logging.error("Exception in execute_json():")
        finally:
            self.fptr.close()  # Закрытие ДТО
        logging.debug(f"execute_json() > None")
        return None

    def is_connected(self, timeout=5):
        logging.debug("ККТ is_connected() timeout %s", timeout)
        connected = False

        def check_connection():
            nonlocal connected
            self.fptr.open()
            if self.fptr.isOpened():
                self.fptr.close()
                logging.debug("ККТ is_connected() True")
                connected = True
            self.fptr.close()

        timer = threading.Timer(timeout, check_connection)
        timer.start()

        while not connected and timer.is_alive():
            time.sleep(0.1)

        timer.cancel()

        logging.debug("ККТ is_connected() %s", connected)
        return connected

    def wait_to_connect(self, timeout=300):
        logging.debug(f"wait_to_connect() timeout={timeout}")
        start_time = time.time()
        while not self.is_connected() and not time.time() - start_time > timeout:
            logging.debug(
                f"Не получен ответ от ККТ, time {time.time() - start_time} timeout {timeout}"
            )
        if self.is_connected():
            logging.debug("ККТ доступна")
            return True
        logging.debug("ККТ не отвечает")
        return False

    def print_info_report(self):
        self.fptr.open()
        self.fptr.setParam(IFptr.LIBFPTR_PARAM_REPORT_TYPE, 5)
        self.fptr.setParam(IFptr.LIBFPTR_PARAM_REPORT_ELECTRONICALLY, 0)
        self.fptr.report()

    def get_TPG_count(self) -> int:
        """
        Получить ресурс ТПГ (постоянный)
        """
        self.fptr.open()
        self.fptr.setParam(IFptr.LIBFPTR_PARAM_DATA_TYPE, IFptr.LIBFPTR_DT_TERMAL_RESOURCE)
        self.fptr.setParam(IFptr.LIBFPTR_PARAM_COUNTER_TYPE, IFptr.LIBFPTR_CT_ROLLUP)
        self.fptr.queryData()
        count = self.fptr.getParamInt(IFptr.LIBFPTR_PARAM_COUNT)

        if count == 0:
            self.print_info_report()

        return count

    def get_TPG_temperature(self):
        """
        Получить температуру ТПГ
        """
        self.fptr.open()

        self.fptr.setParam(IFptr.LIBFPTR_PARAM_DATA_TYPE, IFptr.LIBFPTR_DT_PRINTER_TEMPERATURE)
        self.fptr.queryData()

        temperature = self.fptr.getParamString(IFptr.LIBFPTR_PARAM_PRINTER_TEMPERATURE)
        return temperature

    def get_TPG_motor_count(self):
        """
        Получить ресурс шагового двигателя
        """
        self.fptr.open()

        self.fptr.setParam(IFptr.LIBFPTR_PARAM_DATA_TYPE, IFptr.LIBFPTR_DT_STEP_RESOURCE)
        self.fptr.setParam(IFptr.LIBFPTR_PARAM_COUNTER_TYPE, IFptr.LIBFPTR_CT_ROLLUP)
        self.fptr.setParam(IFptr.LIBFPTR_PARAM_STEP_COUNTER_TYPE, IFptr.LIBFPTR_SCT_OVERALL)
        self.fptr.queryData()

        count = self.fptr.getParamInt(IFptr.LIBFPTR_PARAM_COUNT)

        return count

    def get_registration_data(self):
        """
        Метод запрашивает информацию о регистрации и записывает в атрибуты класса
        organization	Информация об организации	object
        name	Название организации (тег 1048)	string
        vatin	ИНН организации (тег 1018)	string
        email	E-mail организации (адрес отравителя электронных чеков) (тег 1117)	string
        address	Адрес расчетов (тег 1009)	string
        taxationTypes	Список систем налогообложения (тег 1062)	string[]	osn - общая
        usnIncome - упрощенная доход
        usnIncomeOutcome - упрощенная доход минус расход
        esn - единый сельскохозяйственный налог
        patent - патентная система налогообложения
        agents	Признак агента (тег 1057)	string[]	bankPayingAgent - банковский платежный агент
        bankPayingSubagent - банковский платежный субагент
        payingAgent - платежный агент
        payingSubagent - платежный субагент
        attorney - поверенный
        commissionAgent - комиссионер
        another - другой тип агента	по умолчанию - пустой список (не устанавливать флаги агентов)

        device	Параметры ККТ	object
        registrationNumber	Регистрационный номер ККТ (тег 1037)	string
        fnsUrl	Адрес сайта ФНС (тег 1060)	string
        offlineMode	Признак автономного режима (тег 1002)	bool
        machineInstallation	Признак установки принтера в автомате (тег 1221)	bool
        bso	Признак АС БСО (тег 1110)	bool
        encryption	Признак шифрования (тег 1056)	bool
        autoMode	Признак автоматического режима (тег 1001)	bool
        machineNumber	Номер автомата (тег 1036)	string
        internet	Признак ККТ для расчетов только в Интернет (тег 1108)	bool
        service	Признак расчетов за услуги (тег 1109)	bool
        excise	Продажа подакцизного товара (тег 1207)	bool
        gambling	Признак проведения азартных игр (тег 1193)	bool
        lottery	Признак проведения лотереи (тег 1126)	bool
        defaultTaxationType	СНО по умолчанию (тег 1055)	string	osn - общая
        usnIncome - упрощенная доход
        usnIncomeOutcome - упрощенная доход минус расход
        esn - единый сельскохозяйственный налог
        patent - патентная система налогообложения	Можно не указывать, если taxationTypes имеет 1 элемент, иначе должен входить в список taxationTypes
        ofdChannel	Канал обмена с ОФД	string	usb - USB (EoU)
        ethernet - Ethernet
        wifi - Wi-Fi
        gsm - GSM-модем
        proto - средствами протокола ККТ
        tcpipOsStack - TCP/IP стек ОС
        ffdVersion	Версия ФФД (тег 1209)	string	1.05
                                                    1.1
                                                    1.2
        paymentsAddress	Место расчетов (тег 1187)	string
        insurance	Признак осуществления страховой деятельности	bool
        marking	Признак торговли маркированными товарами	bool
        pawnShop	Признак осуществления ломбардной деятельности	bool

        ofd	Параметры ОФД	object
        name	Название ОФД (тег 1046)	string
        vatin	ИНН ОФД (тег 1017)	string
        host	Адрес ОФД	string
        port	Порт ОФД	number	0 - 65535
        dns	DNS ОФД	string
        """

        registration_info = self.execute_json(json_task={
            "type": "getRegistrationInfo"
        })

        try:

            # device
            self.device = registration_info['device']
            self.autoMode = registration_info['device']['autoMode']
            self.bso = registration_info['device']['bso']
            self.defaultTaxationType = registration_info['device']['defaultTaxationType']
            self.encryption = registration_info['device']['encryption']
            self.excise = registration_info['device']['excise']
            self.ffdVersion = registration_info['device']['ffdVersion']
            self.fnsUrl = registration_info['device']['fnsUrl']
            self.gambling = registration_info['device']['gambling']
            self.internet = registration_info['device']['internet']
            self.lottery = registration_info['device']['lottery']
            self.machineInstallation = registration_info['device']['machineInstallation']
            self.machineNumber = registration_info['device']['machineNumber']
            self.ofdChannel = registration_info['device']['ofdChannel']
            self.offlineMode = registration_info['device']['offlineMode']
            self.paymentsAddress = registration_info['device']['paymentsAddress']
            self.registrationNumber = registration_info['device']['registrationNumber']
            self.service = registration_info['device']['service']

            # ofd
            self.ofd = registration_info['ofd']
            self.dns = registration_info['ofd']['dns']
            self.host = registration_info['ofd']['host']
            self.name = registration_info['ofd']['name']
            self.port = registration_info['ofd']['port']
            self.vatin = registration_info['ofd']['vatin']

            # organization
            self.organization = registration_info['organization']
            self.address = registration_info['organization']['address']
            self.agents = registration_info['organization']['agents']
            self.email = registration_info['organization']['email']
            self.name = registration_info['organization']['name']
            self.taxationTypes = registration_info['organization']['taxationTypes']
            self.vatin = registration_info['organization']['vatin']

        except KeyError as error:
            logging.error(f"dto.get_registration_data() ERROR: {error}")

    def get_fn_data(self):
        """
        Метод запрашивает информацию о ФН и записывает в атрибуты класса.
        Информация о ФН (fnInfo)
        serial	Заводской номер ФН	string
        version	Версия ФН	string
        execution	Исполнение ФН (только для ФН-М)	string
        numberOfRegistrations	Количество проведенных регистраций	number
        registrationsRemaining	Количество оставшихся регистраций	number
        validityDate	Срок действия ФН	string
        ffdVersion	Версия ФФД ККТ	string
        fnFfdVersion	Версия ФФД ФН	string
        livePhase	Фаза жизни ФН.
        Принимает значения:
        init - настройка ФН
        configured - настроен, готов в активации
        fiscalMode - фискальный режим
        postFiscalMode - постфискальный режим
        accessArchive - доступ к архиву ФН
        unknown - неизвестная фаза жизни	string
        Предупреждения ФН (fnInfo.warnings)
        memoryOverflow	Память ФН переполнена	bool
        needReplacement	Требуется срочная замена ФН	bool
        ofdTimeout	Превышено время ожидания ответа от ОФД	bool
        resourceExhausted	Исчерпан ресурс ФН	bool
        criticalError	Критическая ошибка ФН	bool
        fnContainsKeysUpdaterServerUri	ФН содержит URI сервера ОКП	bool
        """
        fn_info = self.execute_json(json_task={
            "type": "getFnInfo"
        })
        try:
            self.fnExecution = fn_info['fnInfo']['execution']
            self.FfdVersion = fn_info['fnInfo']['ffdVersion']
            self.fnContainsKeysUpdaterServerUri = fn_info['fnInfo']['fnContainsKeysUpdaterServerUri']
            self.fnFfdVersion = fn_info['fnInfo']['fnFfdVersion']
            self.fnLivePhase = fn_info['fnInfo']['livePhase']
            self.fnNumberOfRegistrations = fn_info['fnInfo']['numberOfRegistrations']
            self.fnRegistrationsRemaining = fn_info['fnInfo']['registrationsRemaining']
            self.fnSerial = fn_info['fnInfo']['serial']
            self.fnValidityDate = fn_info['fnInfo']['validityDate']
            self.fnVersion = fn_info['fnInfo']['version']
            self.fnWarnings = fn_info['fnInfo']['warnings']
        except KeyError as error:
            logging.error(f"dto.get_fn_data() ERROR: {error}")

    def fiscalisation(self, reg_json: dict):
        """
        Фискализация ККТ
        """
        logging.info("Фискализация ККТ")
        self.execute_json(json_task=reg_json, timeout=180)
